/*****************************************************************************
* Copyright (c) Bridgetek Pte Ltd.
* Software License Agreement
*
* This code is provided as an example only and is not guaranteed by Bridgetek. 
* BRIDGETEK accept no responsibility for any issues resulting from its use. 
* The developer of the final application incorporating any parts of this 
* sample project is responsible for ensuring its safe and correct operation 
* and for any consequences resulting from its use.
*****************************************************************************/
/**
* @file  App_Imageviewer
* @brief Sample application to demonstrate EVE primitives and widgets.
Versoin 5.0 - April/17/2017 - Restructure source code
*/

#include "Platform.h"
#include "App_Common.h"
#include "sdcard.h"

/* Global used for context hal */
Gpu_Hal_Context_t host,*phost;
extern Reader imageFile;

/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */

#define MAX_IMAGES  7
char imagename[MAX_IMAGES][12] = {"autumn.jpg","daisy.jpg","empire.jpg","fred2.jpg","me320.jpg","pencils.jpg","tulips.jpg"};					// all are wav files		

                             
static int jpeg_index;
static dirent jpeg_dirent;
static uint8_t sd_present=0;


/* Apis used to upload the image to GRAM from SD card*/
void Load_Jpeg(Reader &r)
{
  uint8_t imbuff[512];
  uint16_t blocklen;
  while (r.offset < r.size)
  {
    uint16_t n = min(512, r.size - r.offset);
    n = (n + 3) & ~3;   // force 32-bit alignment
    r.readsector(imbuff);
    Gpu_Hal_WrCmdBuf(phost,imbuff,n);				/* copy data continuously into command memory */
  }
}


void Loadimage2ram(uint8_t bmphandle)
{
  static byte image_index = 0,max_files=7;
  byte file;
 
  file = imageFile.openfile(imagename[image_index]); 

  if(file==NULL)
  {
      Gpu_Hal_WrCmd32(phost,CMD_MEMSET);
      Gpu_Hal_WrCmd32(phost,(bmphandle ? 131072L : 100));
      Gpu_Hal_WrCmd32(phost,(bmphandle ? 150 : 100));
      Gpu_Hal_WrCmd32(phost,320L*2*194);      
  }else
  {  
    Gpu_Hal_WrCmd32(phost,CMD_LOADIMAGE);
    Gpu_Hal_WrCmd32(phost,(bmphandle ? 131072L : 100));
    Gpu_Hal_WrCmd32(phost,0); 
    Load_Jpeg(imageFile);
  }  
  image_index++;
  if(image_index > (max_files-1)) image_index = 0;
  return;
}

/* Image viewer API*/
void Imageviewer()
{
  uint16_t nooffiles,i,xv;
  uint16_t aspect_ratio = 0;
  uint16_t transform = 273L,boot=1;
  
  int32_t  px = ((DispWidth-320)/2)+aspect_ratio,
	      x = ((DispWidth-320)/2)+aspect_ratio,
	      tracker=0,temp_x;


  
  uint8_t 
 		r = 1,
		loaded = 0,Touch_detected=0;

  temp_x = x;
  
  Gpu_Hal_Wr16(phost,REG_VOL_SOUND,100);
  Gpu_Hal_Wr16(phost,REG_SOUND,0x51);
 
   
  Loadimage2ram(r);	
 // Compute the gradient 
  for (i = 0; i < 64; i++)
  Gpu_Hal_Wr8(phost,i,96- (3 * i / 2));
 // Set the bitmap properties 
  App_Set_CmdBuffer_Index(0);
  Gpu_CoCmd_Dlstart(phost);    
  App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(r));
  App_WrCoCmd_Buffer(phost, BITMAP_SOURCE((r ? 131072L : 100)));
  App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,320L*2,194));
  App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,320,194));
  App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(2));
  App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(0));
  App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(L8, 1, 64));
  App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST, REPEAT, BORDER, DispWidth,64));
  
  App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
  App_WrCoCmd_Buffer(phost,VERTEX2II(x, (10+aspect_ratio),r, 0));	
  App_WrCoCmd_Buffer(phost,DISPLAY());
  Gpu_CoCmd_Swap(phost);
  App_Flush_Co_Buffer(phost); 
  Gpu_Hal_WaitCmdfifo_empty(phost);	 
  while(1)
  {
    Gpu_CoCmd_Dlstart(phost);  
    App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
    Gpu_CoCmd_Gradient(phost,0, DispHeight/2, 0x000000, 0, DispHeight, 0x605040);
    App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
    if (x != temp_x)
    {
      App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(r ^ 1));
      App_WrCoCmd_Buffer(phost,CELL(0));
      App_WrCoCmd_Buffer(phost,VERTEX2F(16 * (x-400), 16 *(10+aspect_ratio)));
    }
    // Image reflection on the floor
    
    App_WrCoCmd_Buffer(phost,VERTEX2II(x, (10+aspect_ratio),r, 0));	
    App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
    App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
    App_WrCoCmd_Buffer(phost,BLEND_FUNC(ONE, ZERO));
    #ifdef SAMAPP_DISPLAY_QVGA
    App_WrCoCmd_Buffer(phost,VERTEX2II(x, 207, 2, 0));
    #else
    App_WrCoCmd_Buffer(phost,VERTEX2II(0, 212, 2, 0));
    #endif
    App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
    App_WrCoCmd_Buffer(phost,BLEND_FUNC(DST_ALPHA, ONE_MINUS_DST_ALPHA));

    Gpu_CoCmd_LoadIdentity(phost);
    Gpu_CoCmd_Translate(phost,(temp_x)*65536L, 65536L*96.5);
    Gpu_CoCmd_Scale(phost,1*65536, 65536*-1);
    Gpu_CoCmd_Translate(phost,-(temp_x)*65536L, 65536L*-96.5);
    Gpu_CoCmd_SetMatrix(phost);    
    if (x != temp_x)
    {
	App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(r ^ 1));
	App_WrCoCmd_Buffer(phost,CELL(0));
	App_WrCoCmd_Buffer(phost,VERTEX2F(16 * (x-400), 16 * 212));
    }
    #ifdef SAMAPP_DISPLAY_QVGA
    App_WrCoCmd_Buffer(phost,VERTEX2II(x, 207, r, 0));
    #else
    App_WrCoCmd_Buffer(phost,VERTEX2II(x, 212, r, 0));
    #endif
    App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());		
    if(px == temp_x && loaded==0 && boot==0) 
    {  
      App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(r^1));
      App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(((r^1) ? 131072L : 100)));
      App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(RGB565,320L*2,194));
      App_WrCoCmd_Buffer(phost,BITMAP_SIZE(NEAREST,BORDER,BORDER,320,194));
      Loadimage2ram(r ^ 1);
      loaded = 1;
   }

  App_WrCoCmd_Buffer(phost,DISPLAY());
  Gpu_CoCmd_Swap(phost);
  App_Flush_Co_Buffer(phost);
  Gpu_Hal_WaitCmdfifo_empty(phost);	
  boot = 0; 
  /* read the tracker*/ 
  tracker = Gpu_Hal_Rd32(phost,REG_TOUCH_SCREEN_XY);
  /* change the bitmap handle if touch is detected*/
  if(loaded==1 && x == temp_x && tracker!=0x80008000)
  {
    Gpu_Hal_Wr8(phost,REG_PLAY,1);	// Play the Sound	
    x = DispWidth; 
    r ^= 1;
    loaded = 0;	
  }
  xv = 1 + x / 16;
  px = x;
  x = max(temp_x, x - xv);
 }
}


#if defined(ARDUINO_PLATFORM)
prog_char8_t * const info[] =
#else
char *info[] =
#endif 
{  "EVE ImageViewer Application",
    "APP to demonstrate interactive Jpeg decode,",
    "using Blend function, Bitmap flip,", 
    "& Jpeg decode"
}; 

#if defined MSVC_PLATFORM | defined FT9XX_PLATFORM
/* Main entry point */
int32_t main(int32_t argc,char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM)||defined(MSVC_FT800EMU)
void setup()
#endif
{
    phost = &host;
    /* Init HW Hal */
    App_Common_Init(phost);
    /* Show Logo, do calibration and display welcome screeen */
    App_Common_Start(phost,info);
    /* Main application */
    Imageviewer();    
    /* Close all the opened handles */
    App_Common_Close(phost);

#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */























